<?php
/**
 * MVCLight
 *
 * An open source application development framework for PHP
 *
 * @package		MVCLight
 * @author		Jorge Albaladejo Pomares
 * @copyright	Copyright (c) 2011, Jorge Albaladejo Pomares
 * @license		http://jorgealbaladejo.com/portfolio/MVCLight/license.txt
 * @link		http://jorgealbaladejo.com/MVCLight
 * @since		Version 0.1a
 * @filesource
 */

/**
 * system/lib/baseObject.class.php
 * 
 * Abstract parent base object from which controllers and models inherit
 * 
 */
Abstract Class baseObject 
{
	/**
	 * Object registry
	 * 
	 * @access protected
	 *
	 */
	protected $registry;
	
	/**
	 * Object router
	 * 
	 * @access protected
	 *
	 */
	protected $router;
	
	/**
	 * Keeps a reference to the user session for further use
	 * 
	 * @access protected
	 */
	protected $session;
	
	/**
	 * Object template
	 * 
	 * @access protected
	 *
	 */
	protected $template;
	
	/**
	 * Object Input
	 *
	 * @access protected
	 *
	 */
	 protected $input;
	 
	/**
	 * Object Language
	 *
	 * @access protected
	 *
	 */
	 protected $language;
	
	/**
	 * Default constructor, loads the registry (singleton pattern)
	 * 
	 */
	public function __construct() 
	{
		$this->registry = Registry::getInstance();
		$this->router	= $this->registry->router;
		$this->template	= $this->registry->template;
		$this->session 	= $this->registry->session;
		$this->input	= $this->registry->input;
		$this->language	= $this->registry->language;	
	}
	
	/**
	 * Loads a model from application or system folders
	 * 
	 * @param string $model name
	 * 
	 * Adds the model as an attribute of $this
	 * 
	 */
	public function model($model)
	{
		$file 	 = __APP_PATH . '/models/' . $model . 'Model.php';
		$adapter = __APP_PATH . '/adapters/' . $model . 'Adapter.php';
		
		if (!is_readable($file))
		{
			$file 	 = __SYSTEM_PATH . '/lib/models/' . $model . 'Model.php';
		}
		
		if (!is_readable($adapter))
		{
			$adapter = __SYSTEM_PATH . '/lib/adapters/' . $model . 'Adapter.php';
		}
		
		if (is_readable($file))
		{
			include_once ($file);
			
			// include adapter if exists
			if (is_readable($adapter))
			{
				include_once ($adapter);				
			}
			
			$modelName = $model . 'Model';
			$this->$model = new $modelName;
		}
		
	}
	
 	/**
	 * Loads a helper from application or system folders
	 * 
	 * @param string $helper name
	 * 
	 * Just loads the file and the functions are available
	 * 
	 */
	public function helper($helper)
	{
		$file = __APP_PATH . '/helpers/' . $helper . '.php';
		
		if (!is_readable($file))
		{
			$file = __SYSTEM_PATH . '/helpers/' . $helper . '.php';
		}
		
		if (is_readable($file))
		{
			include ($file);
		}
		
	}
	

}

?>
