<?php
/**
 * MVCLight
 *
 * An open source application development framework for PHP
 *
 * @package		MVCLight
 * @author		Jorge Albaladejo Pomares
 * @copyright	Copyright (c) 2011, Jorge Albaladejo Pomares
 * @license		http://jorgealbaladejo.com/portfolio/MVCLight/license.txt
 * @link		http://jorgealbaladejo.com/MVCLight
 * @since		Version 0.1a
 * @filesource
 */

/**
 * system/lib/router.class.php
 * 
 * The router class used to execute the desired controller/method
 * 
 */
class router
{
	/**
	 * the controller path
	 * 
	 * @access private
	 */
	private $path;
	
	/**
	 * array of arguments
	 * 
	 * @access private
	 * 
	 */
	private $args = array();
	
	/**
	 * the file to load
	 * 
	 * @access private
	 * 
	 */
	private $file;
	
	/**
	 * the controller name to use
	 * 
	 * @access private
	 * 
	 */
	private $controllerName;
	
	/**
	 * A reference to the controller being executed
	 * 
	 * @access private
	 * 
	 */
	private $controller;
	
	/**
	 * the action to execute
	 * 
	 * @access private
	 * 
	 */
	private $action; 
	
	/**
     * Router object provides storage for shared objects.
     * 
     * @var Router
     * 
     */
    private static $singleton = null;
    
    /**
     * The initialization time, for profiling purposes
     * 
     * @access public
     * 
     */
    public $loadTime = null;
    
    /**
     * The execution starting time, for profiling purposes
     * 
     * @access public
     * 
     */
    public $executeTime = null;
    
    /**
     * Keeps a reference to registry
     * 
     * @access private
     */
    private $registry = null;
	
	/**
	 * Constructs and loads router and controller path
	 * 
	 * @return void
	 * 
	 */
	private function __construct() 
	{
		$this->registry = Registry::getInstance();
		
	    // if profiler is activated, load microtime
        if ($this->registry->get('enable_profiler'))
        {
        	Profiler::addLog('loading router');
        }
	}
	
	/**
	 * The main destructor erases singleton object
	 * 
	 * @return void
	 */
	public function __destruct() 
	{  			
      	self::$singleton = null ;
  	}
	
	/**
	 * __clone private so nobody can clone the instance
	 *
	 * @access private
	 */
	private function __clone(){}
	
	/**
     * Retrieves the default router instance.
     *
     * @return Router
     * 
     */
    public static function getInstance()
    {
        if (self::$singleton === null) 
        {
            self::$singleton = new Router();
        }

        return self::$singleton;
    }
	
	/**
	 *
	 * @set controller directory path
	 *
	 * @param string $path
	 *
	 * @return void
	 *
	 */
	function setPath($path) 
	{
		// check if path is a directory
		if (is_dir($path) == false)
		{
			throw new Exception ('Invalid controller path: "' . $path . '"');
		}
		// set the path
	 	$this->path = $path;
	}
	
	/**
	 *
	 * @load the controller
	 *
	 * @access public
	 *
	 * @return void
	 *
	 */
	public function loader()
	{
		// check the route
		$this->parseRoute();
	
		// if the file is not there diaf
		if (is_readable($this->file) == false)
		{
			$this->file 			= $this->path . '/errorController.php';
	        $this->controllerName 	= 'error';
		}
	
		// include the controller
		include $this->file;
	
		// a new controller class instance
		$class 				= $this->controllerName . 'Controller';
		$this->controller 	= new $class();
	
		// check if the action is callable
		if (is_callable(array($this->controller, $this->action)) == false)
		{
			$this->action = 'index';
		}
		
		// if profiler is activated, show some debug info
        if ($this->registry->get('enable_profiler'))
        {
        	Profiler::addLog('launching action');
        	Profiler::checkpoint('total to launch action');
        	Profiler::setAction($this->action);
        	Profiler::setController($this->controllerName);
        }
		
		// run the action
		$action = $this->action;
		$this->controller->$action();
		
		// if profiler is activated, show some debug info
        if ($this->registry->get('enable_profiler'))
        {
        	//Profiler::checkpoint('executing action');
        	Profiler::finish('total execution time');
        	Profiler::show();
        }
        
	}
		
	/**
	 *
	 * @get the controller
	 *
	 * @access private
	 *
	 * @return void
	 *
	 */
	private function parseRoute() 
	{
		// get the route from the url
		$route = $this->registry->input->get('rt');
		if ( empty( $route ) )
		{
		 	$route = $this->registry->default_route;	
		}	
		
		// get the parts of the route
		$parts = explode('/', $route);
		$this->controllerName = $parts[0];
		if(isset( $parts[1]))
		{
			$params = explode('?', $parts[1]);
			$this->action = $params[0];
		}
		
		// set the file path
		$this->file = $this->path .'/'. $this->controllerName . 'Controller.php';
		
		// custom error controller?
		if (!file_exists($this->file))
		{
			$this->controllerName = 'error';
			$this->action = 'index';
			
			$this->file = $this->path .'/'. $this->controllerName . 'Controller.php';
			
			if (!file_exists($this->file))
			{
				$this->file = __SYSTEM_PATH . '/lib/'. $this->controllerName . 'Controller.class.php';	
			}			
		}
		
		return;
	}
	
	/**
	 * Basic getter for $this->action
	 * 
	 * @return string $this->action
	 * 
	 * @access public
	 * 
	 */
	public function getAction()
	{
		return $this->action;
	}
	
	/**
	 * Basic getter for $this->controllerName
	 * 
	 * @return string $this->controllerName
	 * 
	 * @access public
	 * 
	 */
	public function getController()
	{
		return $this->controllerName;
	}
}

?>
